@@ -308,15 +308,107 @@ Here is a simple example of how on_pause() should be used::
     Both `on_pause` and `on_stop` must save important data because after
     `on_pause` is called, `on_resume` may not be called at all.
 
+Asynchronous app
+----------------
+
+In addition to running an app normally,
+Kivy can be run within an async event loop such as provided by the standard
+library asyncio package or the trio package (highly recommended).
+
+Background
+~~~~~~~~~~
+
+Normally, when a Kivy app is run, it blocks the thread that runs it until the
+app exits. Internally, at each clock iteration it executes all the app
+callbacks, handles graphics and input, and idles by sleeping for any remaining
+time.
+
+To be able to run asynchronously, the Kivy app may not sleep, but instead must
+release control of the running context to the asynchronous event loop running
+the Kivy app. We do this when idling by calling the appropriate functions of
+the async package being used instead of sleeping.
+
+Async configuration
+~~~~~~~~~~~~~~~~~~~
+
+To run a Kivy app asynchronously, either the :func:`async_runTouchApp` or
+:meth:`App.async_run` coroutine must be scheduled to run in the event loop of
+the async library being used.
+
+The environmental variable ``KIVY_EVENTLOOP`` or the ``async_lib`` parameter in
+:func:`async_runTouchApp` and :meth:`App.async_run` set the async
+library that Kivy uses internally when the app is run with
+:func:`async_runTouchApp` and :meth:`App.async_run`. It can be set to one of
+`"asyncio"` when the standard library `asyncio` is used, or `"trio"` if the
+trio library is used. If the environment variable is not set and ``async_lib``
+is not provided, the stdlib ``asyncio`` is used.
+
+:meth:`~kivy.clock.ClockBaseBehavior.init_async_lib` can also be directly
+called to set the async library to use, but it may only be called before the
+app has begun running with :func:`async_runTouchApp` or :meth:`App.async_run`.
+
+To run the app asynchronously, one schedules :func:`async_runTouchApp`
+or :meth:`App.async_run` to run within the given library's async event loop as
+in the examples shown below. Kivy is then treated as just another coroutine
+that the given library runs in its event loop. Internally, Kivy will use the
+specified async library's API, so ``KIVY_EVENTLOOP`` or ``async_lib`` must
+match the async library that is running Kivy.
+
+
+For a fuller basic and more advanced examples, see the demo apps in
+``examples/async``.
+
+Asyncio example
+~~~~~~~~~~~~~--
+
+.. code-block:: python
+
+    import asyncio
+
+    from kivy.app import async_runTouchApp
+    from kivy.uix.label import Label
+
+
+    loop = asyncio.get_event_loop()
+    loop.run_until_complete(
+        async_runTouchApp(Label(text='Hello, World!'), async_lib='asyncio'))
+    loop.close()
+
+Trio example
+~~~~~~~~~~--
+
+.. code-block:: python
+
+    import trio
+
+    from kivy.app import async_runTouchApp
+    from kivy.uix.label import Label
+
+    trio.run(async_runTouchApp, Label(text='Hello, World!'), async_lib='trio')
+
+Interacting with Kivy app from other coroutines
+-----------------------------------------------
+
+It is fully safe to interact with any kivy object from other coroutines
+running within the same async event loop. This is because they are all running
+from the same thread and the other coroutines are only executed when Kivy
+is idling.
+
+Similarly, the kivy callbacks may safely interact with objects from other
+coroutines running in the same event loop. Normal single threaded rules apply
+to both case.
+
+.. versionadded:: 2.0.0
+
 '''
 
-__all__ = ('App', )
+__all__ = ('App', 'runTouchApp', 'async_runTouchApp', 'stopTouchApp')
 
 import os
 from inspect import getfile
 from os.path import dirname, join, exists, sep, expanduser, isfile
 from kivy.config import ConfigParser
-from kivy.base import runTouchApp, stopTouchApp
+from kivy.base import runTouchApp, async_runTouchApp, stopTouchApp
 from kivy.compat import string_types
 from kivy.factory import Factory
 from kivy.logger import Logger
@@ -463,6 +555,9 @@ class App(EventDispatcher):
     __events__ = ('on_start', 'on_stop', 'on_pause', 'on_resume',
                   'on_config_change', )
 
+    # Stored so that we only need to determine this once
+    _user_data_dir = ""
+
     def __init__(self, **kwargs):
         App._running_app = self
         self._app_directory = None
@@ -617,19 +712,13 @@ class App(EventDispatcher):
             return resource_find(self.icon)
 
     def get_application_config(self, defaultpath='%(appdir)s/%(appname)s.ini'):
-        '''.. versionadded:: 1.0.7
-
-        .. versionchanged:: 1.4.0
-            Customized the default path for iOS and Android platforms. Added a
-            defaultpath parameter for desktop OS's (not applicable to iOS
-            and Android.)
-
+        '''
         Return the filename of your application configuration. Depending
         on the platform, the application file will be stored in
         different locations:
 
             - on iOS: <appdir>/Documents/.<appname>.ini
-            - on Android: /sdcard/.<appname>.ini
+            - on Android: <user_data_dir>/.<appname>.ini
             - otherwise: <appdir>/<appname>.ini
 
         When you are distributing your application on Desktops, please
@@ -649,12 +738,24 @@ class App(EventDispatcher):
         - The tilda '~' will be expanded to the user directory.
         - %(appdir)s will be replaced with the application :attr:`directory`
         - %(appname)s will be replaced with the application :attr:`name`
+
+        .. versionadded:: 1.0.7
+
+        .. versionchanged:: 1.4.0
+            Customized the defaultpath for iOS and Android platforms. Added a
+            defaultpath parameter for desktop OS's (not applicable to iOS
+            and Android.)
+
+        .. versionchanged:: 1.11.0
+            Changed the Android version to make use of the
+            :attr:`~App.user_data_dir` and added a missing dot to the iOS
+            config file name.
         '''
 
         if platform == 'android':
-            defaultpath = '/sdcard/.%(appname)s.ini'
+            return join(self.user_data_dir, '.{0}.ini'.format(self.name))
         elif platform == 'ios':
-            defaultpath = '~/Documents/%(appname)s.ini'
+            defaultpath = '~/Documents/.%(appname)s.ini'
         elif platform == 'win':
             defaultpath = defaultpath.replace('/', sep)
         return expanduser(defaultpath) % {
@@ -737,6 +838,29 @@ class App(EventDispatcher):
                 self._app_directory = '.'
         return self._app_directory
 
+    def _get_user_data_dir(self):
+        # Determine and return the user_data_dir.
+        data_dir = ""
+        if platform == 'ios':
+            data_dir = expanduser(join('~/Documents', self.name))
+        elif platform == 'android':
+            from jnius import autoclass, cast
+            PythonActivity = autoclass('org.kivy.android.PythonActivity')
+            context = cast('android.content.Context', PythonActivity.mActivity)
+            file_p = cast('java.io.File', context.getFilesDir())
+            data_dir = file_p.getAbsolutePath()
+        elif platform == 'win':
+            data_dir = os.path.join(os.environ['APPDATA'], self.name)
+        elif platform == 'macosx':
+            data_dir = '~/Library/Application Support/{}'.format(self.name)
+            data_dir = expanduser(data_dir)
+        else:  # _platform == 'linux' or anything else...:
+            data_dir = os.environ.get('XDG_CONFIG_HOME', '~/.config')
+            data_dir = expanduser(join(data_dir, self.name))
+        if not exists(data_dir):
+            os.mkdir(data_dir)
+        return data_dir
+
     @property
     def user_data_dir(self):
         '''
@@ -753,30 +877,27 @@ class App(EventDispatcher):
         On iOS, `~/Documents/<app_name>` is returned (which is inside the
         app's sandbox).
 
-        On Android, `/sdcard/<app_name>` is returned.
-
         On Windows, `%APPDATA%/<app_name>` is returned.
 
         On OS X, `~/Library/Application Support/<app_name>` is returned.
 
         On Linux, `$XDG_CONFIG_HOME/<app_name>` is returned.
+
+        On Android, `Context.GetFilesDir
+        <https://developer.android.com/reference/android/content/\
+Context.html#getFilesDir()>`_ is returned.
+
+        .. versionchanged:: 1.11.0
+
+            On Android, this function previously returned
+            `/sdcard/<app_name>`. This folder became read-only by default
+            in Android API 26 and the user_data_dir has therefore been moved
+            to a writeable location.
+
         '''
-        data_dir = ""
-        if platform == 'ios':
-            data_dir = join('~/Documents', self.name)
-        elif platform == 'android':
-            data_dir = join('/sdcard', self.name)
-        elif platform == 'win':
-            data_dir = os.path.join(os.environ['APPDATA'], self.name)
-        elif platform == 'macosx':
-            data_dir = '~/Library/Application Support/{}'.format(self.name)
-        else:  # _platform == 'linux' or anything else...:
-            data_dir = os.environ.get('XDG_CONFIG_HOME', '~/.config')
-            data_dir = join(data_dir, self.name)
-        data_dir = expanduser(data_dir)
-        if not exists(data_dir):
-            os.mkdir(data_dir)
-        return data_dir
+        if self._user_data_dir == "":
+            self._user_data_dir = self._get_user_data_dir()
+        return self._user_data_dir
 
     @property
     def name(self):
@@ -791,9 +912,7 @@ class App(EventDispatcher):
             self._app_name = clsname.lower()
         return self._app_name
 
-    def run(self):
-        '''Launches the app in standalone mode.
-        '''
+    def _run_prepare(self):
         if not self.built:
             self.load_config()
             self.load_kv(filename=self.kv_file)
@@ -823,9 +942,26 @@ class App(EventDispatcher):
             return
 
         self.dispatch('on_start')
+
+    def run(self):
+        '''Launches the app in standalone mode.
+        '''
+        self._run_prepare()
         runTouchApp()
         self.stop()
 
+    async def async_run(self, async_lib=None):
+        '''Identical to :meth:`run`, but is a coroutine and can be
+        scheduled in a running async event loop.
+
+        See :mod:`kivy.app` for example usage.
+
+        .. versionadded:: 2.0.0
+        '''
+        self._run_prepare()
+        await async_runTouchApp(async_lib=async_lib)
+        self.stop()
+
     def stop(self, *largs):
         '''Stop the application.
 
@@ -839,6 +975,7 @@ class App(EventDispatcher):
         if self._app_window:
             for child in self._app_window.children:
                 self._app_window.remove_widget(child)
+        App._running_app = None
 
     def on_start(self):
         '''Event handler for the `on_start` event which is fired after
